package ActiveState::Milter::SockSpec;

use strict;
use base 'Exporter';

use Carp qw(croak);

our @EXPORT_OK = qw(parse is_valid is_in_conflict);

sub parse {
    # imitate the way the connection spec is parsed in
    # libmilter's mi_milteropen() function
    my $conn = shift;
    $conn = "" unless defined $conn;

    my $scheme;
    if ($conn =~ s/^([^:]*)://) {
	my $s = lc($1);
	if ($s eq "" || $s eq "unix" || $s eq "local") {
	    $scheme = "unix";
	}
	elsif ($s eq "inet" || $s eq "inet6") {
	    $scheme = $s;
	}
	else {
	    croak("Unknown protocol scheme '$s'");
	}
    }
    else {
	$scheme = "unix";
    }
    if ($scheme eq "unix") {
	# the rest is the path
	croak("No path given") unless length $conn;
	return ($scheme, $conn);
    }
    else {
	# port[@interface]
	my $if;
	if ($conn =~ s/\@(.*)//) {
	    $if = $1;
	    if ($scheme eq "inet") {
		require Socket;
		my $tmp = Socket::inet_aton($if);
		croak("Can't resolve '$if'") unless $tmp;
		$if = Socket::inet_ntoa($tmp);
	    }
	}

	my $port = $conn;
	if ($port =~ /^\d/) {
	    $port = int($port);
	}
	else {
	    my $tmp = getservbyname($port, "tcp");
	    croak("Unknown port name '$port'") unless $tmp;
	    $port = $tmp;
	}
	return($scheme, $port, $if);
    }
}

sub is_valid {
    my $conn = shift;
    eval { parse($conn) };
    return !$@;
}

sub is_in_conflict {
    my %conns;
    for (@_) {
	my @c = parse($_);
	push(@{$conns{shift @c}}, \@c);
    }

    if (my $u = $conns{unix}) {
	# similar path
	my %seen;
	my @paths = map $_->[0], @$u;
	@paths = sort { length($b) <=> length($a) } @paths;
	for my $p (@paths) {
	    return 1 if $seen{$p};
	      while ($p =~ m,/,) {
		  $seen{$p}++;
		  $p =~ s,/[^/]*\z,,;
	      }
	}
    }

    my %seen;
    for my $scheme (keys %conns) {
	next if $scheme eq "unix";
	# sort them by port
	for (@{$conns{$scheme}}) {
	    my($port, $if) = @$_;
	    if (defined $if) {
		return 1 if $seen{$port}{$if};
		return 1 if $seen{$port}{"*"};
	    }
	    else {
		return 1 if $seen{$port};
		$if = "*";
	    }
	    
	    $seen{$port}{$if}++;
	}

    }

    return 0;
}

1;
